@@ -220,10 +220,14 @@ def _task(ast, scene, parent, states):
 def _output(ast, scene, parent, states):
     ''' Create an OUTPUT or PROCEDURE CALL symbol '''
     _, _ = scene, states
-    if ast.kind == 'output':
-        return sdlSymbols.Output(parent, ast=ast)
-    elif ast.kind == 'procedure_call':
-        return sdlSymbols.ProcedureCall(parent, ast=ast)
+    return sdlSymbols.Output(parent, ast=ast)
+
+
+@render.register(ogAST.ProcedureCall)
+def _output(ast, scene, parent, states):
+    ''' Create an OUTPUT or PROCEDURE CALL symbol '''
+    _, _ = scene, states
+    return sdlSymbols.ProcedureCall(parent, ast=ast)
 
 
 @render.register(ogAST.Decision)
