@@ -19,17 +19,18 @@
     Contact: maxime.perrotin@esa.int
 """
 
-import math
-import PySide
-from PySide import QtGui, QtCore
-import genericSymbols
 import logging
+import math
+from PySide import QtGui, QtCore
+
 try:
     import pygraphviz as dotgraph
 except ImportError:
     print('pygraphviz not found. Statechart module will not work\n'
-            'If you are using Debian (or Ubuntu) you can install it from\n'
-            'the official repos: sudo apt-get install python-pygraphviz')
+          'If you are using Debian (or Ubuntu) you can install it from\n'
+          'the official repos: sudo apt-get install python-pygraphviz')
+
+import genericSymbols
 
 RENDER_DPI = {'X': 93.0, 'Y': 95.0}
 G_SYMBOLS = set()
@@ -65,7 +66,6 @@ class Record(genericSymbols.HorizontalSymbol, object):
         path = QtGui.QPainterPath()
         path.moveTo(0, 25)
         path.lineTo(width, 25)
-        # check if adding mode = Qt.RelativeSize would not be better
         path.addRoundedRect(0, 0, width, height, 15, 15)
         self.setPath(path)
         super(Record, self).set_shape(width, height)
@@ -76,34 +76,12 @@ class Record(genericSymbols.HorizontalSymbol, object):
 
     def __str__(self):
         ''' User-friendly information about the node '''
-        return('Node ' + self.name + ' at pos ' + str(self.pos()) +
-                ' bounding rect = ' + str(self.boundingRect()))
-
-    def mouse_move(self, event):
-        ''' Handle mouse move: update the edge on the fly '''
-        super(Record, self).mouse_move(event)
-        delta_pos = event.pos() - event.lastPos()
-        delta = [delta_pos.x(), delta_pos.y()]
-        for edge in edges(self.scene(), self):
-            if edge.edge['source'] == self:
-                #edge.edge['start'] = [
-                #        sum(u) for u in zip(edge.edge['spline'][0], delta)]
-                #edge.edge['spline'][0] = edge.edge['start']
-                edge.end_point -= delta_pos
-            if edge.edge['target'] == self:
-                #edge.edge['end'] = [
-                #        sum(u) for u in zip(edge.edge['end'], delta)]
-                edge.end_point += delta_pos
-            #edge.end_point = [
-            #       sum(u) for u in zip(edge.edge['end'], delta)]
-
-                #edge.edge['spline'][-1] = edge.edge['end']
-            edge.reshape()
+        return('Record ' + self.name + ' at pos ' + str(self.pos()) +
+               ' bounding rect = ' + str(self.boundingRect()))
 
     def mouse_release(self, _):
         ''' After moving item, ask dot to recompute the edges '''
-        #update(self.scene())
-        # Discard mouse release default action
+        # Discard mouse release default action (CAM, UndoCommand)
         pass
 
 
@@ -162,7 +140,6 @@ class Diamond(genericSymbols.HorizontalSymbol, object):
         super(Diamond, self).__init__(x=node['pos'][0],
                 y=node['pos'][1], text='')
         self.set_shape(node['width'], node['height'])
-        #self.setBrush(QtGui.QBrush(QtCore.Qt.black))
         self.graph = graph
 
     def set_shape(self, width, height):
@@ -187,21 +164,22 @@ class Diamond(genericSymbols.HorizontalSymbol, object):
 
     def mouse_release(self, _):
         ''' After moving item, ask dot to recompute the edges '''
-        #self.scene().refresh()
-        update(self.scene())
+        #update(self.scene())
+        pass
 
 
 class Controlpoint(QtGui.QGraphicsPathItem, object):
-    ''' Class handling one connection control point (mouse movements) '''
-    def __init__(self, pos, parent):
+    ''' Class handling one edge control point (to change bezier curves) '''
+    def __init__(self, pos, edge):
         ''' Set the original control point - with color, shape '''
         path = QtGui.QPainterPath()
         path.addEllipse(pos.x() - 5, pos.y() - 5, 10, 10)
-        super(Controlpoint, self).__init__(path, parent)
+        super(Controlpoint, self).__init__(path, parent=edge)
         self.setPen(QtGui.QColor(50, 100, 120, 200))
         self.setBrush(QtGui.QColor(200, 200, 210, 120))
         self.setFlags(QtGui.QGraphicsItem.ItemIsSelectable |
-                QtGui.QGraphicsItem.ItemIsMovable)
+                      QtGui.QGraphicsItem.ItemIsMovable)
+        self.edge = edge
         self.hide()
 
     @property
@@ -212,7 +190,55 @@ class Controlpoint(QtGui.QGraphicsPathItem, object):
     def mouseMoveEvent(self, event):
         ''' When user moves a control point, update the connection shape '''
         super(Controlpoint, self).mouseMoveEvent(event)
-        self.parentItem().reshape()
+        self.edge.reshape()
+
+
+class Connectionpoint(Controlpoint, object):
+    '''
+        Class handling an end connection point - similar to Controlpoint,
+        except the shape and the behaviour when moved. Connection point
+        must check with its owning symbol if the move is valid.
+    '''
+    def __init__(self, pos, edge, symbol):
+        ''' Create the point - as a small, lightblue box '''
+        super(Connectionpoint, self).__init__(pos, edge=edge)
+        path = QtGui.QPainterPath()
+        path.addRect(0, 0, 10, 10)
+        self.setPath(path)
+        self.setPos(pos.x() - 5, pos.y() - 5)
+        # Symbol actually owning the connection point
+        self.symbol = symbol
+
+
+    def update_position(self):
+        '''
+            Update the position of the end point so that it is always
+            placed correctly on the symbol that contains it (but that may
+            not be the parent of the point if is the end point)
+            After the position update, reshape the edge so that the lines
+            are properly drawn to the new position
+        '''
+        # Transform position to owning symbol coordinates:
+        center_pos = self.scenePos() + self.boundingRect().center()
+        pos_symb = self.symbol.mapFromScene(center_pos)
+
+        # Ask to symbol the nearest valid position on its shape
+        # to place the connection point, and adjust position
+        nearest_x, nearest_y = self.symbol.closest_connection_point(pos_symb)
+
+        self.moveBy(-nearest_x, -nearest_y)
+
+        self.edge.reshape()
+
+    def mouseMoveEvent(self, event):
+        '''
+            When user moves the point, check with the connected symbol
+            where the actual connection point should be placed
+        '''
+        # calling super function moves to the new point
+        super(Connectionpoint, self).mouseMoveEvent(event)
+        # Then compute a valid position, based on the owning symbol
+        self.update_position()
 
 
 class Edge(genericSymbols.Connection, object):
@@ -239,6 +265,14 @@ class Edge(genericSymbols.Connection, object):
                           for j in range(3)])
         # Set connection points as not visible, by default
         self.bezier_visible = False
+        
+        # Create connection points at start and end of the edge
+        self.source_connection = Connectionpoint(
+                self.start_point or self.bezier[0], self, self.parent)
+        self.parent.movable_points.append(self.source_connection)
+        self.end_connection = Connectionpoint(
+                self.end_point or self.bezier[-1], self, self.child)
+        self.child.movable_points.append(self.end_connection)
         self.reshape()
 
     def bezier_set_visible(self, visible=True):
@@ -250,6 +284,12 @@ class Edge(genericSymbols.Connection, object):
                     ctrl_point.show()
                 else:
                     ctrl_point.hide()
+        if visible:
+            self.end_connection.show()
+            self.source_connection.show()
+        else:
+            self.end_connection.hide()
+            self.source_connection.hide()
         self.update()
 
     def mousePressEvent(self, event):
@@ -262,17 +302,17 @@ class Edge(genericSymbols.Connection, object):
         path = QtGui.QPainterPath()
         # If there is a starting point, draw a line to the first curve point
         if self.start_point:
-            path.moveTo(self.start_point)
+            path.moveTo(self.source_connection.center) #start_point)
             path.lineTo(self.bezier[0])
         else:
-            path.moveTo(self.bezier[0])
+            path.moveTo(self.source_connection.center)#bezier[0])
         # Loop over the curve points:
         for group in self.bezier[1:]:
             path.cubicTo(*[point.center for point in group])
 
         # If there is an ending point, draw a line to it
         if self.end_point:
-            path.lineTo(self.end_point)
+            path.lineTo(self.end_connection.center) #end_point)
 
         # Draw the arrow head
         length = path.length()
@@ -297,6 +337,7 @@ class Edge(genericSymbols.Connection, object):
         path.lineTo(arrow_p2)
         path.moveTo(end_point)
         try:
+            # Add the transition label, if any (none for the START edge)
             font = QtGui.QFont('arial', pointSize=8)
             width = QtGui.QFontMetrics(font).width(
                     self.edge.get('label', 0))
@@ -332,8 +373,8 @@ class Edge(genericSymbols.Connection, object):
             painter.setBrush(QtCore.Qt.NoBrush)
             points_flat = [point.center
                            for sub1 in self.bezier[1:] for point in sub1]
-            painter.drawPolyline([self.bezier[0]] + points_flat +
-                                 [self.end_point])
+            painter.drawPolyline([self.source_connection.center]
+                                  + points_flat + [self.end_connection.center])
 
 def edges(scene, node):
     ''' Return all edges of a given node '''
