@@ -195,7 +195,7 @@ class EditableText(QGraphicsTextItem, object):
 
     def __init__(self, parent, text='...', hyperlink=None):
         super(EditableText, self).__init__(parent)
-        self.setFont(QFont('Ubuntu', 8))
+        self.setFont(QFont('Ubuntu', 10))
         self.completer = Completer(self)
         self.completer.widget().itemActivated.connect(
                 self.completion_selected)
@@ -1124,11 +1124,6 @@ class Comment(Symbol, object):
             local_pos = parent.mapFromScene(ast.pos_x, ast.pos_y)
             self.insert_symbol(parent, local_pos.x(), local_pos.y())
         self.set_shape(ast.width, ast.height)
-        pen = QPen()
-        # Set transparent color (drawing is done in the paint function)
-        # FIXME : should not be needed anymore using QPainterPath
-        pen.setColor(QColor(0, 0, 0, 0))
-        self.setPen(pen)
         self.text = EditableText(parent=self, text=ast.inputString,
                 hyperlink=ast.hyperlink)
         self.common_name = 'end'
