@@ -39,93 +39,197 @@ LOG = logging.getLogger(__name__)
 
 
 class Expression(object):
-    ''' AST Entry for expressions '''
-    def __init__(self, inputString, line, charPositionInLine):
+    ''' AST Entry for expressions - Always use subtype '''
+    is_raw = False
+    def __init__(self, inputString='', line=-1, charPositionInLine=-1):
         ''' Initialize Expression attributes '''
         self.inputString = inputString
         self.line = line
         self.charPositionInLine = charPositionInLine
-        # left and right are of type Expression (absent if kind=='primary')
+        # left and right are of type Expression (absent for primaries)
         self.left = None
         self.right = None
         # exprType is an ASN.1 type (as exported by asn1scc)
         self.exprType = None
-        # kind can be 'primary', in that case value is in 'var',
-        # or one of 'plus' 'mul' 'minus' 'or' 'and' 'xor' 'eq' 'neq' 'gt' 'ge'
-        # 'lt' 'le' 'div' 'mod' 'append' 'in' 'rem', 'assign'
-        self.kind = None
-        # var is of type Primary
-        self.var = None
         # Hint for code generators: intermediate storage identifier
         self.tmpVar = -1
 
     def __repr__(self):
         ''' Debug output for an expression '''
-        return '{k}: {exp} ({l},{c})'.format(exp=self.inputString,
-                k=self.kind.upper(), l=self.line, c=self.charPositionInLine)
-
-    def is_raw(self):
-        ''' Helper function - check if an expression is a terminal primary '''
-        return (self.kind == 'primary'
-                and self.var.kind not in ('expression', 'constant')
-                and (self.var.kind != 'primaryId' or
-                    (len(self.var.primaryId) < 2
-                     and self.exprType.kind != 'ReferenceType')))
+        return '{exp} ({l},{c})'.format(exp=self.inputString,
+                                        l=self.line,
+                                        c=self.charPositionInLine)
 
 
-class Primary(object):
-    ''' AST entry for a primary construction '''
-    def __init__(self, inputString, line, charPositionInLine):
-        ''' Initialize primary attributes (see comments below for details) '''
-        self.inputString = inputString
-        self.line = line
-        self.charPositionInLine = charPositionInLine
-        self.op_not = False
-        self.op_minus = False
-        # kind can be 'primaryId', 'enumeratedValue', 'numericalValue_int',
-        # 'numericalValue_float',
-        # 'booleanValue', 'expression', 'ifThenElse', 'stringLiteral',
-        # 'mantissaBaseExpFloat','emptyString' (SEQUENCE OF type value {}),
-        # 'choiceItem', 'sequenceOf', 'sequence', 'choiceDeterminant',
-        # 'constant'
-        self.kind = None
-        # primType is the ASN.1 type of the primary
-        self.primType = None
-        # primaryId is a list of elements needed to identify a value
-        # for example, "i!j!k(5)" is stored as:
-        # [ 'i', 'j', 'k', {'index':[Expression list]}]
-        #    (in that case, 5 is an index)
-        # other example: "hello(world)" ->
-        #      ['hello', {'procParams':[Expression list]'}]
-        # (in that case, hello is an operator and world is its parameter)
-        # primaryId is set only when kind ==
-        #'primaryId', 'enumeratedValue' or 'numericalValue', 'booleanValue'
-        self.primaryId = []
-        # expr is an Expression
-        self.expr = None
-        # ifThenElse is a dict:
-        # { 'if': Expression, 'then': Expression,
-        # 'else': Expression, 'tmpVar': integer}
-        # tmpVar can be used if the backend needs a temporary variable
-        # to process the ifThenElse
-        self.ifThenElse = None
-        # stringLiteral is a string (in e.g. a := 'Hello',
-        #    'Hello' is the string literal. Quotes are kept)
-        self.stringLiteral = None
-        # mantissaBaseExpFloat is a dict:
-        # {'mantissa': int, 'base': int, 'exponent': int}
-        self.mantissaBaseExpFloat = None
-        # choiceItem is a dict : {'choice': string, 'value': Expression}
-        self.choiceItem = {}
-        # sequenceOf is a list of Primary
-        self.sequenceOf = []
-        # sequence is a dict : { field1: Expression, field2: Expression }
-        self.sequence = {}
+class ExprPlus(Expression):
+    operand = '+'
+
+
+class ExprMul(Expression):
+    operand = '*'
+
+
+class ExprMinus(Expression):
+    operand = '-'
+
+
+class ExprOr(Expression):
+    operand = 'or'
+
+
+class ExprAnd(Expression):
+    operand = 'and'
+
+
+class ExprXor(Expression):
+    operand = 'xor'
+
+
+class ExprEq(Expression):
+    operand = '='
+
+
+class ExprNeq(Expression):
+    operand = '/='
+
+
+class ExprGt(Expression):
+    operand = '>'
+
+
+class ExprGe(Expression):
+    operand = '>='
+
+
+class ExprLt(Expression):
+    operand = '<'
+
+
+class ExprLe(Expression):
+    operand = '<='
+
+
+class ExprDiv(Expression):
+    operand = '/'
+
+
+class ExprMod(Expression):
+    operand = 'mod'
+
+
+class ExprRem(Expression):
+    operand = 'mod'
+
+
+class ExprAssign(Expression):
+    operand = ':='
+
+
+class ExprAppend(Expression): pass
+class ExprIn(Expression): pass
+class ExprImplies(Expression): pass
+
+class Primary(Expression):
+    '''
+        AST entry for a primary construction (that is a terminal expression)
+        Abstract class, never used directly, see subtypes below
+    '''
+    is_raw = True
+    def __init__(self, inputString='', line=-1, charPositionInLine=-1,
+                 primary=None):
+        ''' Initialize common primary attributes '''
+        if primary:
+            self.inputString = primary.inputString
+            self.line = primary.line
+            self.charPositionInLine = primary.charPositionInLine
+            self.op_not = primary.op_not
+            self.op_minus = primary.op_minus
+            self.value = primary.value
+            self.exprType = primary.exprType
+        else:
+            self.inputString = inputString
+            self.line = line
+            self.charPositionInLine = charPositionInLine
+            self.op_not = False
+            self.op_minus = False
+            self.value = None
+            self.exprType = None
 
     def __repr__(self):
         ''' Debug output for a primary '''
-        return 'PRIMARY {k}: {exp} ({l},{c})'.format(exp=self.inputString,
-                k=self.kind, l=self.line, c=self.charPositionInLine)
+        return 'PRIMARY : {exp} ({l},{c})'.format(exp=self.inputString,
+                l=self.line, c=self.charPositionInLine)
+
+# Subclasses of Primary - never use Primary directly
+class PrimPath(Primary):
+    ''' PrimPath is a list of elements needed to identify a value
+        For example, "i!j!k(5)" is stored as:
+        [ 'i', 'j', 'k', {'index':[Expression list]}]
+           (in that case, 5 is an index)
+        other example: "hello(world)" ->
+             ['hello', {'procParams':[Expression list]'}]
+        (in that case, hello is an operator and world is its parameter)
+    '''
+    is_raw = False
+
+class PrimVariable(PrimPath): pass # XXX should not be raw for codegen
+class PrimEnumeratedValue(Primary): pass
+class PrimInteger(Primary): pass
+class PrimReal(Primary): pass
+class PrimBoolean(Primary): pass
+class PrimConstant(Primary): is_raw = False #pass
+class PrimBitStringLiteral(Primary): pass   # Not supported yet
+class PrimOctetStringLiteral(Primary): pass # Not supported yet
+
+
+class PrimIfThenElse(Primary):
+    ''' value is a dictionnary:
+        { 'if': Expression, 'then': Expression,
+        'else': Expression, 'tmpVar': integer}
+        tmpVar can be used if the backend needs a temporary variable
+        to process the ifThenElse
+    '''
+    pass
+
+
+class PrimStringLiteral(Primary):
+    ''' Value is a string with quotes '''
+    @property
+    def exprType(self):
+        return self._exprType
+
+    @exprType.setter
+    def exprType(self, val):
+#       name = getattr(self, 'value', self.inputString)
+#       import traceback
+#       traceback.print_stack()
+#       print 'SET type of', name, 'to', val
+        self._exprType = val
+
+
+class PrimMantissaBaseExp(Primary):
+    ''' Value is a dict: {'mantissa': int, 'base': int, 'exponent': int} '''
+    pass
+
+
+class PrimEmptyString(Primary): pass
+
+
+class PrimChoiceItem(Primary):
+    ''' Value is a dict : {'choice': string, 'value': Expression} '''
+    pass
+
+
+class PrimChoiceDeterminant(Primary): pass
+
+
+class PrimSequenceOf(Primary):
+    ''' Value is a list of Primary '''
+
+
+class PrimSequence(Primary):
+    ''' Value is a dict : { field1: Expression, field2: Expression } '''
+    pass
 
 
 class Decision(object):
@@ -180,7 +284,7 @@ class Answer(object):
         # constant is an Expression
         #    (contains 'open_range' and 'constant' kinds corresponding value)
         self.constant = None
-        # one of 'eq' 'neq' 'gt' 'ge 'lt' 'le'
+        # one of ExprEq, ExprNeq, ExprGt, ExprGe, ExprLt, ExprLe (types)
         self.openRangeOp = None
         # transition is of type Transition
         self.transition = None
@@ -206,20 +310,11 @@ class Task(object):
         self.pos_y = 0
         self.width = 70
         self.height = 35
-        # one of 'informal_text', 'assign', 'for_loop'
-        self.kind = None
-        # list of 'assign'-kind Expression
-        self.assign = []
-        # list of strings
-        self.informalText = []
-        # list of for-loops:{'var': string, 'type': Asn1SccType,
-        # 'list': Primary, 'range': {'start': expression, 'stop': expression,
-        # 'step' : int}, 'transition': Transition }
-        self.forloop = []
         # optional comment symbol
         self.comment = None
         # optional hyperlink
         self.hyperlink = None
+        self.elems = []
 
     def __repr__(self):
         ''' Debug output for a task '''
@@ -227,6 +322,24 @@ class Task(object):
                 l=self.line, c=self.charPositionInLine)
 
 
+class TaskAssign(Task):
+    ''' 'elems' contains a list of ExprAssign expressions '''
+    pass
+
+
+class TaskInformalText(Task):
+    ''' 'elems' contains a list of strings '''
+    pass
+
+
+class TaskForLoop(Task):
+    '''
+        'elems' is a list of for-loops:{'var': string, 'type': Asn1SccType,
+        'list': Primary, 'range': {'start': expression, 'stop': expression,
+        'step' : int}, 'transition': Transition }
+    '''
+    pass
+
 class Output(object):
     ''' AST Entry for OUTPUT statements '''
     def __init__(self, defName='RI'):
@@ -238,8 +351,6 @@ class Output(object):
         self.height = 35
         self.line = None
         self.charPositionInLine = None
-        # one of 'procedure_call', 'output'
-        self.kind = None
         # list of {'outputName':ID, 'params':list of Expression, 'tmpVars':[]}
         self.output = []
         # optional comment symbol
@@ -249,8 +360,13 @@ class Output(object):
 
     def __repr__(self):
         ''' Debug output for an Output symbol '''
-        return '{kind} {exp} ({l},{c})'.format(exp=self.inputString,
-                kind=self.kind.upper(), l=self.line, c=self.charPositionInLine)
+        return '{exp} ({l},{c})'.format(exp=self.inputString,
+                l=self.line, c=self.charPositionInLine)
+
+
+class ProcedureCall(Output):
+    ''' Procedure calls content is identical to Outputs '''
+    pass
 
 
 class Terminator(object):
@@ -310,11 +426,19 @@ class Label(object):
 
 class Floating_label(Label, object):
     ''' AST Entry for a floating label '''
-    def __init__(self):
+    def __init__(self, label=None):
         ''' Initialize the floating label attributes '''
         super(Floating_label, self).__init__()
-        # Transition following the label - other attribues inherited from Label
-        self.transition = None
+        if label:
+            self.inputString = label.inputString
+            self.pos_x = label.pos_x
+            self.pos_y = label.pos_y
+            self.width = label.width
+            self.height = label.height
+            self.line = label.line
+            self.charPositionInLine = label.charPositionInLine
+            self.terminators = label.terminators
+            self.transition = label.transition
 
     def __repr__(self):
         ''' Debug output for a label (used by code generator) '''
