@@ -81,10 +81,6 @@ class Input(HorizontalSymbol, object):
         if ast.comment:
             Comment(parent=self, ast=ast.comment)
 
-    def __str__(self):
-        ''' Return the text string as entered by the user '''
-        return str(self.text)
-
     def check_syntax(self):
         ''' Redefined function, to check only the symbol, not recursively '''
         _, syntax_errors, ___, ____, _____ = self.parser.parseSingleElement(
@@ -107,7 +103,7 @@ class Input(HorizontalSymbol, object):
         ''' Compute the polygon to fit in width, height '''
         path = QPainterPath()
         path.lineTo(width, 0)
-        path.lineTo(width - 15, height / 2)
+        path.lineTo(width - 11, height / 2)
         path.lineTo(width, height)
         path.lineTo(0, height)
         path.lineTo(0, 0)
@@ -163,16 +159,12 @@ class Output(VerticalSymbol, object):
         if ast.comment:
             Comment(parent=self, ast=ast.comment)
 
-    def __str__(self):
-        ''' Return the text string as entered by the user '''
-        return str(self.text)
-
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
         path = QPainterPath()
-        path.lineTo(width - 15, 0)
+        path.lineTo(width - 11, 0)
         path.lineTo(width, height / 2)
-        path.lineTo(width - 15, height)
+        path.lineTo(width - 11, height)
         path.lineTo(0, height)
         path.lineTo(0, 0)
         self.setPath(path)
@@ -236,10 +228,6 @@ class Decision(VerticalSymbol, object):
         return (branch for branch in self.childSymbols()
                 if isinstance(branch, DecisionAnswer))
 
-    def __str__(self):
-        ''' Return the text string as entered by the user '''
-        return str(self.text)
-
     def check_syntax(self):
         ''' Redefined function, to check only the symbol, not recursively '''
         _, syntax_errors, ___, ____, _____ = self.parser.parseSingleElement(
@@ -390,10 +378,6 @@ class DecisionAnswer(HorizontalSymbol, object):
         self.branch_entrypoint = self
         self.parser = ogParser
 
-    def __str__(self):
-        ''' Return the text string as entered by the user '''
-        return str(self.text)
-
     def check_syntax(self):
         ''' Redefined function, to check only the symbol, not recursively '''
         _, syntax_errors, ___, ____, _____ = self.parser.parseSingleElement(
@@ -420,9 +404,10 @@ class DecisionAnswer(HorizontalSymbol, object):
     def set_shape(self, width, height):
         ''' ANSWER has round, disjoint sides - does not fit in a polygon '''
         self.width, self.height = width, height
+        point = width / 2.85
         path = QPainterPath()
-        left = QRect(0, 0, 35, height)
-        right = QRect(width - 35, 0, 35, height)
+        left = QRect(0, 0, point, height)
+        right = QRect(width - point, 0, point, height)
         path.arcMoveTo(left, 125)
         path.arcTo(left, 125, 110)
         path.arcMoveTo(right, -55)
@@ -468,10 +453,10 @@ class Join(VerticalSymbol, object):
 
     def __init__(self, parent=None, ast=None):
         if not ast:
-            ast = ogAST.Terminator(defName='Goto')
+            ast = ogAST.Terminator(defName='Go')
             ast.pos_y = 0
-            ast.width = 50
-            ast.height = 50
+            ast.width = 35
+            ast.height = 35
         super(Join, self).__init__(parent, text=ast.inputString,
                 x=ast.pos_x, y=ast.pos_y, hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
@@ -479,10 +464,6 @@ class Join(VerticalSymbol, object):
         self.terminal_symbol = True
         self.parser = ogParser
 
-    def __str__(self):
-        ''' Return the text string as entered by the user '''
-        return str(self.text)
-
     def resize_item(self, rect):
         ''' Redefinition of the resize item (block is a square) '''
         size = min(rect.width(), rect.height())
@@ -519,8 +500,8 @@ class ProcedureStop(Join, object):
         if not ast:
             ast = ogAST.Terminator(defName='ret')
             ast.pos_y = 0
-            ast.width = 50
-            ast.height = 50
+            ast.width = 35
+            ast.height = 35
         super(ProcedureStop, self).__init__(parent, ast)
     completion_list = set()
 
@@ -577,9 +558,9 @@ class Label(VerticalSymbol, object):
         self.textbox_alignment = Qt.AlignLeft | Qt.AlignTop
         self.parser = ogParser
 
-    def __str__(self):
-        ''' Return the text string as entered by the user '''
-        return str(self.text)
+    @property
+    def common_name(self):
+        return 'label' if self.hasParent else 'floating_label'
 
     def set_shape(self, width, height):
         ''' Define the shape of the LABEL symbol '''
@@ -622,14 +603,18 @@ class Label(VerticalSymbol, object):
         return '\n'.join(result)
 
     def __repr__(self):
-        ''' Return the PR string for a standard label '''
-        pos = self.scenePos()
-        return ('/* CIF LABEL ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                '{label}:'.format(label=str(self), hlink=repr(self.text),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height())))
+        ''' Return the PR string for a label '''
+        if self.common_name == 'floating_label':
+            return self.parse_gr()
+        else:
+            # Standard label
+            pos = self.scenePos()
+            return ('/* CIF LABEL ({x}, {y}), ({w}, {h}) */\n'
+                    '{hlink}'
+                    '{label}:'.format(label=str(self), hlink=repr(self.text),
+                        x=int(pos.x()), y=int(pos.y()),
+                        w=int(self.boundingRect().width()),
+                        h=int(self.boundingRect().height())))
 
 
 # pylint: disable=R0904
@@ -658,10 +643,6 @@ class Task(VerticalSymbol, object):
         if ast.comment:
             Comment(parent=self, ast=ast.comment)
 
-    def __str__(self):
-        ''' Return the text string as entered by the user '''
-        return str(self.text)
-
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
         path = QPainterPath()
@@ -711,10 +692,6 @@ class ProcedureCall(VerticalSymbol, object):
         if ast.comment:
             Comment(parent=self, ast=ast.comment)
 
-    def __str__(self):
-        ''' Return the text string as entered by the user '''
-        return str(self.text)
-
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
         path = QPainterPath()
@@ -772,10 +749,6 @@ class TextSymbol(HorizontalSymbol, object):
                     'text_area', repr(self))
         Task.completion_list |= {dcl for dcl in ast.variables.keys()}
 
-    def __str__(self):
-        ''' Return the text string as entered by the user '''
-        return str(self.text)
-
     def set_shape(self, width, height):
         ''' Define the polygon of the text symbol '''
         path = QPainterPath()
@@ -854,10 +827,6 @@ class State(VerticalSymbol, object):
         if ast.comment:
             Comment(parent=self, ast=ast.comment)
 
-    def __str__(self):
-        ''' Return the text string as entered by the user '''
-        return str(self.text)
-
     def update_completion_list(self):
         ''' When text was entered, update state completion list '''
         # Get AST for the symbol
@@ -880,7 +849,7 @@ class State(VerticalSymbol, object):
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
         path = QPainterPath()
-        path.addRoundedRect(0, 0, width, height, 15, 50)
+        path.addRoundedRect(0, 0, width, height, height / 4, height)
         self.setPath(path)
         super(State, self).set_shape(width, height)
 
@@ -961,15 +930,15 @@ class Procedure(HorizontalSymbol, object):
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
         path = QPainterPath()
-        path.addRect(10, 0, width-20, height)
-        path.moveTo(10, 0)
-        path.lineTo(0, 10)
-        path.lineTo(0, height - 10)
-        path.lineTo(10, height)
-        path.moveTo(width - 10, 0)
-        path.lineTo(width, 10)
-        path.lineTo(width, height - 10)
-        path.lineTo(width - 10, height)
+        path.addRect(7, 0, width - 14, height)
+        path.moveTo(7, 0)
+        path.lineTo(0, 7)
+        path.lineTo(0, height - 7)
+        path.lineTo(7, height)
+        path.moveTo(width - 7, 0)
+        path.lineTo(width, 7)
+        path.lineTo(width, height - 7)
+        path.lineTo(width - 7, height)
         self.setPath(path)
         super(Procedure, self).set_shape(width, height)
 
@@ -1040,7 +1009,7 @@ class Start(HorizontalSymbol, object):
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
         path = QPainterPath()
-        path.addRoundedRect(0, 0, width, height, 25, 25)
+        path.addRoundedRect(0, 0, width, height, height / 2, height / 2)
         self.setPath(path)
         super(Start, self).set_shape(width, height)
 
@@ -1070,10 +1039,10 @@ class ProcedureStart(Start, object):
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
         path = QPainterPath()
-        path.addRoundedRect(0, 0, width, height, 25, 25)
-        path.moveTo(25, 0)
-        path.lineTo(25, height)
-        path.moveTo(width-25, 0)
-        path.lineTo(width-25, height)
+        path.addRoundedRect(0, 0, width, height, height / 2, height / 2)
+        path.moveTo(min(width / 2, height / 2), 0)
+        path.lineTo(min(width / 2, height / 2), height)
+        path.moveTo(max(width / 2, width - height / 2), 0)
+        path.lineTo(max(width / 2, width - height / 2), height)
         self.setPath(path)
         super(Start, self).set_shape(width, height)
